#define USBCLASS
using HidLibrary;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
#if !USBCLASS
using System.Windows.Forms;
#endif

#if !USBCLASS
namespace TemperTest
#else
namespace TempErLibrary
#endif
{
#if !USBCLASS
    public partial class TemperReader : Form
#else
    public class TemperClass
#endif
    {

        private HidDevice Control;
        private HidDevice Bulk;
        private bool IsInitialised = false;
        private String Manufacturer = String.Empty;
        private String Product = String.Empty;
        private String Serial = String.Empty;
        private Double Calibration_Offset = -1.70;
        private Double Calibration_Scale = 1;
#if USBCLASS
        private Double deg = -100.0;
#endif

        public delegate void ReadHandlerDelegate(HidReport report);
        //commands / magic numbers
        byte[] ini = { 0x01, 0x01 };
        byte[] temp = {0x01, 0x80, 0x33, 0x01, 0x00, 0x00, 0x00, 0x00};
        byte[] ini1 = {0x01, 0x82, 0x77, 0x01, 0x00, 0x00, 0x00, 0x00};
        byte[] ini2 = {0x01, 0x86, 0xff, 0x01, 0x00, 0x00, 0x00, 0x00 };




#if !USBCLASS
        public TemperReader()
#else
        public int TemperReader()
#endif
        {
#if !USBCLASS
            InitializeComponent();
#endif
            HidDevice[] _deviceList = HidDevices.Enumerate().ToArray();
            List<HidDevice> _TemperInterfaces = _deviceList.Where(x => x.Attributes.ProductHexId == "0x7401" & x.Attributes.VendorHexId == "0x0C45").ToList();
            //Temper has two interfaces
            Control = _TemperInterfaces.Find(x => x.DevicePath.Contains("mi_00"));
            Bulk = _TemperInterfaces.Find(x => x.DevicePath.Contains("mi_01"));
#if USBCLASS
	    if(Control == null || Bulk == null)
            {
              return 0;
            }
#endif
            // connect event handlers
            Control.Inserted += Device_Inserted;
            Control.Removed += Device_Removed;
            Control.MonitorDeviceEvents = true;
#if !USBCLASS
            timer1.Tick+= new EventHandler(OnTimedEvent);
            timer1.Enabled = true;
#else
	     return 1;
#endif



        }

        private void ReadBogus(HidReport report)
        {
        }

#if !USBCLASS
        private void ReadTemperatureProcess(HidReport report)
        {
            BeginInvoke(new ReadHandlerDelegate(ReadTemperatureHandler), new object[] { report });
        }
#endif


#if !USBCLASS
        private void ReadTemperatureHandler(HidReport report)
#else
        private void ReadTemperatureHandler(HidReport report)
#endif
        {
            if (IsInitialised)
            {
                int RawReading = (report.Data[3] & 0xFF) + (report.Data[2] << 8);

                double temperatureCelsius = (Calibration_Scale*(RawReading * (125.0 / 32000.0))) + Calibration_Offset;
#if !USBCLASS
                double temperatureFahrenheit = temperatureCelsius * 1.8 + 32.0;
                TemperatureReadingC.Text = temperatureCelsius.ToString("0.00");
                TemperatureReadingF.Text = temperatureFahrenheit.ToString("0.00");
#else
		deg = temperatureCelsius;
		return;
#endif
            }
#if USBCLASS
	    deg = -100.0;
	    return;
#endif
        }
 

        private void Device_Inserted()
        {

            //claim interfaces:
            Control.OpenDevice();
            Bulk.OpenDevice();
            IsInitialised = false;
            byte[] ManufacturerRaw = new byte[64];
            byte[] ProductRaw = new byte[64];
            byte[] SerialRaw = new byte[64];
            
            
            Control.ReadManufacturer(out ManufacturerRaw);
            Control.ReadProduct(out ProductRaw);
            Control.ReadSerialNumber(out SerialRaw);
            Manufacturer = System.Text.Encoding.UTF8.GetString(Encoding.Convert(Encoding.Unicode, Encoding.ASCII, ManufacturerRaw)).TrimEnd('\0');
            Product= System.Text.Encoding.UTF8.GetString(Encoding.Convert(Encoding.Unicode, Encoding.ASCII, ProductRaw)).TrimEnd('\0');
            Serial = System.Text.Encoding.UTF8.GetString(Encoding.Convert(Encoding.Unicode, Encoding.ASCII, SerialRaw)).TrimEnd('\0');
            var outData1 = Control.CreateReport();
            outData1.ReportId = 0x01;
            outData1.Data = ini;
            Control.WriteReport(outData1);
            while (outData1.ReadStatus != HidDeviceData.ReadStatus.Success) ;
            Control.ReadReport(ReadBogus);

            var outData3 = Bulk.CreateReport();
            outData3.ReportId = 0x00;
            outData3.Data = ini1;
            Bulk.WriteReport(outData3);
            while (outData3.ReadStatus != HidDeviceData.ReadStatus.Success) ;
            Bulk.ReadReport(ReadBogus);

            var outData4 = Bulk.CreateReport();
            outData4.ReportId = 0x00;
            outData4.Data = ini2;
            Bulk.WriteReport(outData4);
            while (outData4.ReadStatus != HidDeviceData.ReadStatus.Success) ;
            Bulk.ReadReport(ReadBogus);
            //Clear out garbage from device
            var outData2 = Bulk.CreateReport();
            outData2.ReportId = 0x00;
            outData2.Data = temp;
            Bulk.WriteReport(outData2);
            while (outData2.ReadStatus != HidDeviceData.ReadStatus.Success) ;
            Bulk.ReadReport(ReadBogus);
            Bulk.WriteReport(outData2);
            while (outData2.ReadStatus != HidDeviceData.ReadStatus.Success) ;
            Bulk.ReadReport(ReadBogus);



            IsInitialised = true;
#if !USBCLASS
            timer1.Start();


            if (InvokeRequired)
            {
                BeginInvoke(new Action(Device_Inserted));
                return;
            }
#endif
        }

        private void Device_Removed()
        {
#if !USBCLASS
            timer1.Stop();
#endif
            Bulk.CloseDevice();
            Control.CloseDevice();
            IsInitialised = false;

#if !USBCLASS
            if (InvokeRequired)
            {
                BeginInvoke(new Action(Device_Removed));
                return;
            }
#endif

        }

#if !USBCLASS
        private void OnTimedEvent(object sender, EventArgs e)
        {
            BeginInvoke(new Action(StartTemperatureReading));
        }
#endif

#if !USBCLASS
        private void StartTemperatureReading()
#else
        public double StartTemperatureReading()
#endif
        {
            if (Bulk.IsOpen && IsInitialised)
            {
                var outData = Bulk.CreateReport();
                outData.ReportId = 0x00;
                outData.Data = temp;
                Bulk.WriteReport(outData);
                while (outData.ReadStatus == HidDeviceData.ReadStatus.NoDataRead) ;
#if !USBCLASS
                Bulk.ReadReport(ReadTemperatureProcess);
#else
                Bulk.ReadReport(ReadTemperatureHandler);
		return(deg);
#endif
            }
#if !USBCLASS
            Manufacturer_lbl.Text = Manufacturer;
            Product_lbl.Text = Product;
            Serial_lbl.Text = Serial;
#else
	    deg = -100.0;
	    return(deg);
#endif
        }





 






    }
}
